(function () {

	var app = angular.module('eventosContactApp', [
		'ui.router',
		'ngStorage',
		'pascalprecht.translate',
		'angular-jwt',
		'angularMoment',
		'frapontillo.bootstrap-switch',
		'720kb.tooltips',
		'mgcrea.ngStrap.modal',
	]);

	//	app.filter('unsafe', function($sce) { return $sce.trustAsHtml; });

	//phonegap whitelist
	app.config(['$compileProvider', function ($compileProvider) {
			$compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|ftp|tel|mailto|file|ghttps?|ms-appx|x-wmapp0):/);
			$compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ftp|file|ms-appx|x-wmapp0):|data:image\//);
		}]);

	app.config(function ($locationProvider, $stateProvider, $urlRouterProvider, $localStorageProvider, $httpProvider, jwtOptionsProvider) {

		$localStorageProvider.setKeyPrefix('ec-');
		// For any unmatched url, redirect to home
		$urlRouterProvider.otherwise('/login');

		jwtOptionsProvider.config({
			unauthenticatedRedirectPath: '/login',
			authPrefix: '',
			whiteListedDomains: ['api.eventos2.local', 'api-dev.eventos.fi', 'api-demo.eventos.fi', 'api-staging.eventos.fi', 'api.eventos.fi'],
			tokenGetter: ['options', function (options) {
					// Skip authentication for any requests ending in .html
					if (options && options.url.substr(options.url.length - 5) == '.html') {
						return null;
					}
					return $localStorageProvider.get('jwtToken');
				}]
		});

		$httpProvider.interceptors.push('jwtInterceptor');

		//		if (platform.name === 'IE' || platform.name === 'IE Mobile') { // IE will cache requests without this
		//			//initialize get if not there
		//			if (!$httpProvider.defaults.headers.get) {
		//				$httpProvider.defaults.headers.get = {};
		//			}
		//
		//			// Disable IE ajax request caching using date in the past
		//			$httpProvider.defaults.headers.get['If-Modified-Since'] = 'Mon, 26 Jul 1997 05:00:00 GMT';
		//		}


		var isAuthed = {
			check: function ($q, $state, $localStorage, jwtHelper, $timeout) {
				var deferred = $q.defer();
				$timeout(function () {
					if (typeof $localStorage.jwtToken === 'undefined' || !$localStorage.jwtToken) {
						console.log('User not authorized. Redirecting to login.');
						$state.go('login');
						deferred.reject();
					} else if (jwtHelper.isTokenExpired($localStorage.jwtToken)) {
						$localStorage.selectedEvent = null;
						$localStorage.selectedScanPoint = null;
						console.log('Auth token has expired.');
						$state.go('login');
						deferred.reject();
					} else if($localStorage.jwtToken && !$localStorage.organization && !$localStorage.partner){ // Logged in but organization / partner hasn't been selected
						$localStorage.jwtToken = null;
						$state.go('login');
						deferred.reject();
					} else {
						deferred.resolve();
					}
				});
				return deferred.promise;
			}
		};

		$stateProvider
				.state('login', {
					url: '/login',
					controller: 'loginController',
					templateUrl: 'views/login.html',
					data: {
						pageTitle: 'Login',
						backLink: ''
					}
				})
				.state('app', {
					controller: 'appController',
					templateUrl: 'views/app.html',
					abstract: true
				})
				.state('app.home', {
					url: '/home',
					controller: 'homeController',
					templateUrl: 'views/home.html',
					resolve: isAuthed,
					data: {
						pageTitle: 'Home',
						backLink: ''
					}
				})
				.state('app.selectScanPoint', {
					url: '/selectScanPoint',
					controller: 'selectScanPointController',
					templateUrl: 'views/select-scan-point.html',
					resolve: isAuthed,
					data: {
						pageTitle: 'Select scan point',
						backLink: 'app.home'
					}
				})
				.state('app.selectEvent', {
					url: '/selectEvent',
					controller: 'selectEventController',
					templateUrl: 'views/select-event.html',
					resolve: isAuthed,
					data: {
						pageTitle: 'Select event',
						backLink: 'app.home'
					}
				})
				.state('app.browseScans', {
					url: '/browseScans',
					controller: 'browseScansController',
					templateUrl: 'views/browse-scans.html',
					resolve: isAuthed,
					data: {
						pageTitle: 'Browse scans',
						backLink: ''
					}
				})
				.state('app.activityFeed', {
					url: '/activityFeed',
					controller: 'activityFeedController',
					templateUrl: 'views/activity-feed.html',
					resolve: isAuthed,
					data: {
						pageTitle: 'Activity Feed',
						backLink: ''
					}
				})
				.state('app.participants', {
					url: '/participants',
					controller: 'participantsController',
					templateUrl: 'views/participants.html',
					resolve: isAuthed,
					data: {
						pageTitle: 'Participants',
						backLink: ''
					}
				})
				.state('app.scan', {
					url: '/scan',
					controller: 'scanController',
					templateUrl: 'views/scan.html',
					resolve: isAuthed,
					data: {
						pageTitle: 'Scan',
						backLink: ''
					}
				});
	});

	app.run(function ($rootScope, $localStorage, authManager) {

		$rootScope.ngStorageDefaults = {
			jwtToken: null,
			offlineScans: [],
			failedScans: [],
			selectedEvent: null,
			selectedScanPoint: null,
			organization: null,
			partner: null,
			scanPointSearch: ''
		};
		
		$localStorage.$default($rootScope.ngStorageDefaults);
		
		$.notify.defaults({
//			globalPosition: "top right",
			autoHideDelay: 5000,
			className: 'info'
		});

		$rootScope.connected = true;
		$rootScope.pageTitle = 'Eventos Contact';
		$rootScope.backLink = 'app.home';

		$rootScope.updateConnectionStatus = function () {
			if (typeof navigator.connection.type === "undefined") {
				return;
			}
			var oldConnectionStatus = $rootScope.connected;
			if (navigator.connection.type.toLowerCase() === 'none') {
				console.log("Connection status changed", navigator.connection.type);
				$rootScope.connected = false;
				if ($rootScope.connected !== oldConnectionStatus) {
					$rootScope.$apply();
				}
			} else if (navigator.connection.type.toLowerCase() === 'unknown') {
				console.log("Unknown connection status");
				if ($rootScope.connected !== oldConnectionStatus) {
					$rootScope.$apply();
				}
			} else {
				console.log("Connection status changed", navigator.connection.type);
				$rootScope.connected = true;
				if ($rootScope.connected !== oldConnectionStatus) {
					$rootScope.$apply();
				}
			}
		};

		var onStateChangeSuccess = $rootScope.$on('$stateChangeSuccess',
				function (event, toState, toParams, fromState, fromParams) {

					// On state change scroll window back up
					window.scrollTo(0, 0);

					if (angular.isDefined(toState.data)) {

						var data = toState.data;

						if (angular.isDefined(data.pageTitle)) {
							$rootScope.pageTitle = data.pageTitle;
						}
						if (angular.isDefined(data.backLink)) {
							$rootScope.backLink = data.backLink;
						}
					}
				}
		);

		if (typeof navigator.connection !== "undefined") {
			$rootScope.updateConnectionStatus();
			document.addEventListener("offline", $rootScope.updateConnectionStatus, false);
			document.addEventListener("online", $rootScope.updateConnectionStatus, false);
		}

		$rootScope.$on('tokenHasExpired', function () {
			if ($('.modal-notification').length === 0) {
				var modalNotification = new ModalNotification();
				modalNotification.alert('Your session has expired!');
				$localStorage.selectedEvent = null;
				$localStorage.selectedScanPoint = null;
				$localStorage.jwtToken = null;
			}
		});

		authManager.redirectWhenUnauthenticated();
		authManager.checkAuthOnRefresh();

		$rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams, options) {

		});
	});

})();