/* global angular */

(function () {

	var app = angular.module('eventosContactApp');

	app.controller("appController", ["$scope", "$location", "$rootScope", "$state", "$http", "$localStorage", "$interval", "dataService", "$timeout", "$window", "userService",
		function ($scope, $location, $rootScope, $state, $http, $localStorage, $interval, dataService, $timeout, $window, userService) {

			var modalNotification = new ModalNotification();
		
			$scope.$localStorage = $localStorage;
			$scope.userService = userService;
			$scope.sideMenuOpen = false;
			
			function selectEvent(){
				$scope.events = dataService.events;
				if (!$scope.$localStorage.selectedEvent && $scope.events.length === 1) {
					$scope.$localStorage.selectedEvent = $scope.events[0];
				}
				dataService.getScanPoints();
			}
			function selectScanPoint(){
				$scope.scanPoints = dataService.scanPoints;
				if (!$scope.$localStorage.selectedScanPoint && $scope.scanPoints && $scope.scanPoints.length === 1) {
					$scope.$localStorage.selectedScanPoint = $scope.scanPoints[0];
				}
			}
			if (typeof dataService.events !== 'undefined') {
				selectEvent();
			}
			if (typeof dataService.scanPoints !== 'undefined') {
				selectScanPoint();
			}
			dataService.getEvents();
			$scope.$on('eventsLoaded', selectEvent);
			$scope.$on('scanPointsLoaded', selectScanPoint);
			
			$scope.logOut = function () {
				var logoutMessage = 'Are you sure you want to log out?';
				if ($scope.$localStorage.offlineScans.length > 0) {
					logoutMessage = 'You have offline scans stored on the device. Logging out will destroy the scans. Are you sure you want to log out?';
				}
				modalNotification.confirm({
					title: 'Confirmation',
					cancelText: 'Cancel',
					text: logoutMessage,
					callback: function (response) {
						if (response) {
							$scope.$localStorage.$reset($rootScope.ngStorageDefaults);
							$timeout(function () { // Reload entire app to clear services
								$location.path('/login');
								$window.location.reload();
							});
						}
					}
				});
			};
			
			$scope.isMainMenuLoading = function(){
				if(!$scope.events){ // Still loading events
					return true;
				} else if(!$scope.scanPoints && $scope.$localStorage.selectedEvent){ // Event selected, but still loading scan points
					return true;
				}
				return false;
			};
			
			function sendOfflineScans(offlineScans){
				if($.isEmptyObject(offlineScans)){
					return;
				}
				// Find all scans with same scanPointID as the first one.
				var currentScanPointID = offlineScans[0].scanPointID;
				var scansToSend = [];
				var indexesSent = [];
				for(var i = 0; i < offlineScans.length; i++){
					if(offlineScans[i].scanPointID === currentScanPointID){
						indexesSent.push(i);
						scansToSend.push(offlineScans[i]);
					}
				}
				
				$http.post($rootScope.apiUrl + 'scanPoints/' + currentScanPointID + '/scans/', {scans: scansToSend}).
						then(function (response) {
							console.log('Offline scans sent', response.data);
							for(var i = scansToSend.length -1; i >= 0; i--){
								for(var j = 0; j < $scope.$localStorage.offlineScans.length; j++){
									if($scope.$localStorage.offlineScans[j].identifier === scansToSend[i].identifier && $scope.$localStorage.offlineScans[j].timeScanned === scansToSend[i].timeScanned){
										$scope.$localStorage.offlineScans.splice(j, 1);
									}
								}
							}
						}, function (e) {
							console.log("Error saving scan from offline storage.", e);
							if(typeof e.data === 'undefined' || typeof e.data.data === 'undefined' || typeof e.data.data.scans === 'undefined'){
								return;
							}
							var failedScans = typeof e.data.data.scans.failed === 'undefined' ? [] : e.data.data.scans.failed;
							var successScans = typeof e.data.data.scans.success === 'undefined' ? [] : e.data.data.scans.success;
							var offlineScans = $scope.$localStorage.offlineScans;
							for(var i = 0; i < failedScans.length; i++){
								for(var j = 0; j < offlineScans.length; j++){
									if(failedScans[i].identifier === offlineScans[j].identifier && failedScans[i].timeScanned === offlineScans[j].timeScanned){
										$scope.$localStorage.failedScans.push(failedScans[i]);
										offlineScans.splice(j, 1);
									}
								}
							}
							for(var i = 0; i < successScans.length; i++){
								for(var j = 0; j < offlineScans.length; j++){
									if(successScans[i].personID === offlineScans[j].identifier && successScans[i].timeScanned === offlineScans[j].timeScanned){
										offlineScans.splice(j, 1);
									}
								}
							}
							$scope.$localStorage.offlineScans = offlineScans;
						});
			}

			$interval(function(){
				sendOfflineScans($scope.$localStorage.offlineScans);
				if(typeof dataService.events === 'undefined' && !dataService.dataLoading.events){
					dataService.getEvents();
				}
				if(typeof dataService.scanPoints === 'undefined' && $scope.$localStorage.selectedEvent && !dataService.dataLoading.scanPoints) {
					dataService.getScanPoints();
				}
			}, 30000 * 1);
			
			sendOfflineScans($scope.$localStorage.offlineScans); // Run when app is started.

			$rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams, options) {
				$scope.sideMenuOpen = false;
			});
		}]);
})();
