/* global angular, moment, nfc */

(function () {

	var app = angular.module('eventosContactApp');

	app.controller("browseScansController", function ($scope, $rootScope, $state, $translate, dataService, $http, $localStorage, $modal, userService) {
		console.log("browseScansController loaded");
		
		var modalNotification = new ModalNotification();
		$scope.$localStorage = $localStorage;
		
		$scope.dataService = dataService;
		
		$scope.scanOrder = '-timeScanned';
		$scope.offlineOrder = '-timeScanned';
		$scope.failedOrder = '-timeScanned';

		var editModal = $modal({scope: $scope, templateUrl: 'views/modals/modal-edit-scan-comment.html', show: false, backdrop: 'static'});

		if(!$scope.$localStorage.selectedScanPoint.scanPointID){
			$state.go('app.home');
		}

		function initData(){
			if(typeof dataService.scans !== 'undefined' && dataService.scans !== null && typeof dataService.persons !== 'undefined'){
				$scope.scans = dataService.scans;
				for(var i = 0; i < $scope.scans.length; i++){
					$scope.scans[i].person = dataService.getByID('persons', 'personID', $scope.scans[i].personID);
				}
			}
		}
		dataService.getScans();
		dataService.getPersons();
		initData();
		$scope.$on('scansLoaded', initData);
		$scope.$on('personsLoaded', initData);
		
		$scope.setScanOrder = function(orderItem){
			if(orderItem === $scope.scanOrder.slice(1)){
				var order = $scope.scanOrder.charAt(0) === '-' ? '+' : '-';
				$scope.scanOrder = order + orderItem;
			} else {
				$scope.scanOrder = '-' + orderItem;
			}
		};
		$scope.setOfflineOrder = function(orderItem){
			if(orderItem === $scope.offlineOrder.slice(1)){
				var order = $scope.offlineOrder.charAt(0) === '-' ? '+' : '-';
				$scope.offlineOrder = order + orderItem;
			} else {
				$scope.offlineOrder = '-' + orderItem;
			}
		};
		$scope.setFailedOrder = function(orderItem){
			if(orderItem === $scope.failedOrder.slice(1)){
				var order = $scope.failedOrder.charAt(0) === '-' ? '+' : '-';
				$scope.failedOrder = order + orderItem;
			} else {
				$scope.failedOrder = '-' + orderItem;
			}
		};
		
		$scope.showComment = function(scan){
			if(typeof scan.comment !== 'undefined' && scan.comment){
				modalNotification.alert({
					title: 'Scan comment',
					text: scan.comment
				});
			}
		};			
		
		$scope.editComment = function(scan){
			$scope.editedScan = scan;
			editModal.$promise.then(editModal.show);
		};
		
		$scope.isCommentOpenable = function(scan){
			if(scan.userID === userService.getUser().userID){
				return true;
			} else if(typeof scan.comment !== 'undefined' && scan.comment !== ''){
				return true;
			}
			return false;
		};
		
		$scope.openComment = function(scan){
			if(scan.userID === userService.getUser().userID){
				$scope.editComment(scan);
			} else {
				$scope.showComment(scan);
			}
		};
		
		$scope.saveScanCommentEdit = function(editedScan){
			editModal.$promise.then(editModal.hide);
			$http.post($rootScope.apiUrl + 'scanPoints/scans/' + editedScan.scanPointScanID, {comment: editedScan.comment}).
					then(function (response) {
						$.notify('Scan comment saved.');
						console.log(response.data);
						dataService.getScans();
					}, function (e) {
						console.log("Error saving scan comment edit", e);
					});
		};
	});
})();
