/* global angular */

(function () {

	app = angular.module('eventosContactApp');

	app.controller("loginController", function ($scope, $rootScope, $state, $localStorage, userService, jwtHelper, $modal, $http) {
		
		console.log("loginController loaded");
		
		$scope.$localStorage = $localStorage;
		
		$scope.loginError = false;
		$scope.incorrectPassword = false;
		$scope.loginInProgress = false;
		
		if($scope.$localStorage.jwtToken && !jwtHelper.isTokenExpired($scope.$localStorage.jwtToken)){
			$state.go('app.home');
		}
		
		var loginAsModal = $modal({scope: $scope, templateUrl: 'views/modals/login-as-modal.html', show: false, backdrop: 'static'});
		
		$scope.login = function(loginData){
			$scope.loginInProgress = true;
			$scope.loginError = '';
			userService.login(loginData).then(function (response) {
				$scope.$localStorage.jwtToken = response.data.token;
				$scope.$localStorage.$apply();
				userService.getUserData().then(function (response) {
					$scope.user = response;
					if(response.organizations.length > 0 && response.partners.length > 0){
						loginAsModal.$promise.then(loginAsModal.show);
					} else if(response.organizations.length > 1){
						loginAsModal.$promise.then(loginAsModal.show);
					} else if(response.partners.length > 1){
						loginAsModal.$promise.then(loginAsModal.show);
					} else {
						if(response.organizations.length > 0){
							$scope.$localStorage.organization = response.organizations[0];
						} else if (response.partners.length > 0){
							$scope.$localStorage.partner = response.partners[0];
						}
						$scope.$localStorage.$apply();
						$state.go('app.home');
					}
					$scope.loginInProgress = false;
				}, function(e) {
					if(typeof e.data !== 'undefined' && e.data && typeof e.data.errors !== 'undefined' && e.data.errors && e.data.errors.length > 0){
						$scope.loginError = e.data.errors[0].details;
						console.log('Error trying to get user data', e.data.errors);
					} else {
						$scope.loginError = 'Connection issue! Check your connection.';
						console.log("Error trying to get user data", e);
					}
					$scope.loginInProgress = false;
					// Clear jwtToken if userData fetch failed
					$scope.$localStorage.jwtToken = response.data.token;
					$scope.$localStorage.$apply();
				});
			}, function(e) {
				$scope.loginInProgress = false;
				if(typeof e.data !== 'undefined' && e.data && typeof e.data.errors !== 'undefined' && e.data.errors && e.data.errors.length > 0){
					$scope.loginError = e.data.errors[0].details;
					console.log('Error trying to login', e.data.errors);
				} else {
					$scope.loginError = 'Connection issue! Check your connection.';
					console.log("Error trying to login", e);
				}
			});
		};
		
		$scope.selectOrganization = function (organization) {
			loginAsModal.$promise.then(loginAsModal.hide);
			$http.post($rootScope.apiUrl + 'users/changeOrganization', {organizationID: organization.organizationID, service: 'eventosContact'}).
					then(function (response) {
						$scope.$localStorage.organization = organization;
						$scope.$localStorage.jwtToken = response.data.data.token;
						$scope.$localStorage.$apply();
						$state.go('app.home');
					}, function (e) {
						console.log("Error", e);
					});
		};
		
		$scope.selectPartner = function (partner) {
			loginAsModal.$promise.then(loginAsModal.hide);
			$scope.$localStorage.partner = partner;
			$scope.$localStorage.$apply();
			$state.go('app.home');
		};
	});
})();
