/* global angular */

(function () {

	var app = angular.module('eventosContactApp');

	app.controller("participantsController", function ($scope, $rootScope, $state, dataService, $localStorage, $modal, $http, metaService) {
		console.log("participantsController loaded");
		
		var modalNotification = new ModalNotification();
		$scope.$localStorage = $localStorage;
		$scope.dataService = dataService;
		
		$scope.itemsToShow = 15;
		$scope.loadingData = true;
		
		if(!$scope.$localStorage.selectedEvent){
			$state.go('app.home');
		}
		
		var originalEditedItem;
		var editModal = $modal({scope: $scope, templateUrl: 'views/modals/modal-edit-participant.html', show: false, backdrop: 'static'});
		
		$scope.getPersonStatus = function (statusID){ // TODO Change this to dataService.getByID when API returns statuses as int. Should be next API version. 16.1.2018 Tomi
			for(var i = 0; i < $scope.participantsMetaStatus.length; i++){
				if(statusID == $scope.participantsMetaStatus[i].id){
					return $scope.participantsMetaStatus[i].name;
				}
			}
		};

		function initData(){
			if(typeof dataService.persons !== 'undefined' && typeof dataService.participantsMetaStatus !== 'undefined'){
				if(typeof $scope.persons !== 'undefined'){ // Is refresh
					$.notify('Participants refreshed.');
				}
				$scope.persons = angular.copy(dataService.persons);
				$scope.participantsMetaStatus = dataService.participantsMetaStatus;
				for (var i = 0; i < $scope.persons.length; i++) { // Fetch statusNames to allow using search on them.
					$scope.persons[i].statusName = $scope.getPersonStatus($scope.persons[i].event.personStatus);
				}
				$scope.loadingData = false;
			}
		}
		dataService.getPersons();
		if(typeof dataService.participantsMetaStatus === 'undefined'){
			dataService.getParticipantsMetaStatus();
		}
		initData();
		$scope.$on('participantsMetaStatusLoaded', initData);
		$scope.$on('personsLoaded', initData);
		
		function loadMore(){
			if(typeof $scope.persons === "undefined"){
				return;
			}
			if($scope.persons.length > $scope.itemsToShow){
				$scope.itemsToShow += 5;
				$scope.$digest();
			}
		}
		
		$scope.searchChanged = function(){
			if(typeof $scope.search === 'string'){
				$scope.itemsToShow = 15;
			}
		};
		
		$scope.refreshParticipants = function(){
			$scope.loadingData = true;
			dataService.getPersons();
		};
		
		function personMetaLoaded(){
			$scope.personsMeta = metaService.personsMeta;
			$scope.editedPersonStatus = $scope.editedItem.event.personStatus;
			editModal.$promise.then(editModal.show);
		}
		
		$scope.editParticipant = function (participant) {
			originalEditedItem = typeof participant !== 'undefined' ? participant : {};
			$scope.editedItem = angular.copy(originalEditedItem);

			if(typeof metaService.personsMeta !== 'undefined'){
				personMetaLoaded();
			} else {
				metaService.getPersonsMeta();
				$scope.$on('personsMetaLoaded', personMetaLoaded);
			}
		};
		
		$scope.saveParticipant = function (participant) {
			editModal.$promise.then(editModal.hide);
			$http.post($rootScope.apiUrl + 'events/' + $scope.$localStorage.selectedEvent.eventID  + '/persons/' + participant.personID, participant).
					then(function (response) {
						dataService.getPersons();
					}, function (e) {
						console.log("Error", e);
					});
		};
		
		var $participantsPage;
		
		function scrolled(e){
			if(typeof $participantsPage === 'undefined'){
				$participantsPage = $('#participants-page');
			}
			var bottomOfPage = $participantsPage.position().top + $participantsPage.outerHeight(true);
			if(2 * window.innerHeight + window.pageYOffset > bottomOfPage){
				loadMore();
			}
		}
		
		window.addEventListener('scroll', scrolled);
		
		$scope.$on('$destroy', function () {
			window.removeEventListener('scroll', scrolled, false);
		});
		
	});
})();
