/* global angular, moment, nfc */

(function () {

	var app = angular.module('eventosContactApp');

	app.controller("scanController", function ($scope, $rootScope, $state, $translate, dataService, $http, $localStorage) {
		console.log("scanController loaded");
		
		var modalNotification = new ModalNotification();
		$scope.$localStorage = $localStorage;
		$scope.currentScan = null;
		$scope.nfcSupported = typeof nfc !== 'undefined' ? true : false;
		$scope.continuousScan = false;
		$scope.qrScannerOpen = false;

		if(!$scope.$localStorage.selectedScanPoint){
			$state.go('app.selectScanPoint');
			return;
		}

		function initScanPoints(){
			$scope.scanPoints = dataService.scanPoints;
			for(var i = 0; i < $scope.scanPoints.length; i++){
				if($scope.scanPoints[i].scanPointID === $scope.$localStorage.selectedScanPoint.scanPointID){
					$scope.$localStorage.selectedScanPoint = $scope.scanPoints[i];
					return;
				}
			}
		}
		
		if(typeof dataService.scanPoints !== 'undefined'){
			initScanPoints();
		}
		$scope.$on('scanPointsLoaded', initScanPoints);
		
		dataService.getScanPoints();
		dataService.getPersons();
		
		$scope.openQrScanner = function (force) {
			if(!$scope.isScanPointActive() && (typeof force === 'undefined' || !force)){
				var description = 'Current scan point is not active. Active time for ' + $scope.$localStorage.selectedScanPoint.name + ' is';
				
				if($scope.$localStorage.selectedScanPoint.timeActiveStart !== null){
					description	+= ' from ' + moment($scope.$localStorage.selectedScanPoint.timeActiveStart).format('DD.MM.YYYY hh:mm');
				}
				if($scope.$localStorage.selectedScanPoint.timeActiveEnd !== null){
					description	+= ' until ' + moment($scope.$localStorage.selectedScanPoint.timeActiveEnd).format('DD.MM.YYYY hh:mm');
				}
				description	+= '. Are you sure you want to scan on this scan point?';
				
				modalNotification.confirm({
					title: 'Confirmation',
					cancelText: 'Cancel',
					text: description,
					callback: function (response) {
						if (response) {
							$scope.openQrScanner(true);
						}
					}
				});
				return;
			}
			$scope.qrScannerOpen = true;
			cordova.plugins.barcodeScanner.scan(
					function (result) {
						$scope.qrScannerOpen = false;
						if(result.text !== '' && !result.cancelled){
							if($scope.continuousScan){
								$scope.sendScan({identifier: result.text});
								$scope.openQrScanner();
							} else {
								scanDataReceived(result.text);
							}
						} else {
							console.log('Nothing was scanned.');
						}
						$scope.$apply();
					},
					function (error) {
						$scope.qrScannerOpen = false;
						alert("Scanning failed: " + error);
						$scope.$apply();
					},
					{
						preferFrontCamera: false, // iOS and Android
						showFlipCameraButton: true, // iOS and Android
						showTorchButton: true, // iOS and Android
						torchOn: false, // Android, launch with the torch switched on (if available)
						saveHistory: false, // Android, save scan history (default false)
						prompt: $translate.instant('PLACE_QR_CODE'), // Android
						resultDisplayDuration: 0, // Android, display scanned text for X ms. 0 suppresses it entirely, default 1500
						formats: "QR_CODE", // default: all but PDF_417 and RSS_EXPANDED
						orientation: "portrait", // Android only (portrait|landscape), default unset so it rotates with the device
						disableAnimations: true, // iOS
						disableSuccessBeep: false // iOS and Android
					}
			);
		};
		
		$scope.isScanPointActive = function(){
			var now = moment();
			if($scope.$localStorage.selectedScanPoint.timeActiveStart === null && $scope.$localStorage.selectedScanPoint.timeActiveEnd === null){
				return true;
			} else if($scope.$localStorage.selectedScanPoint.timeActiveStart === null && moment($scope.$localStorage.selectedScanPoint.timeActiveEnd) > now){
				return true;
			} else if($scope.$localStorage.selectedScanPoint.timeActiveEnd === null && moment($scope.$localStorage.selectedScanPoint.timeActiveStart) < now){
				return true;
			} else if(moment($scope.$localStorage.selectedScanPoint.timeActiveStart) < now && moment($scope.$localStorage.selectedScanPoint.timeActiveEnd) > now){
				return true;
			} else {
				return false;
			}
		};
		
		function onTagDiscoevered(nfcEvent){
			if(typeof nfcEvent.tag.ndefMessage === "undefined"){
				modalNotification.alert("Error! NFC tag is empty.");
			} else {
				scanDataReceived(nfc.bytesToString(nfcEvent.tag.ndefMessage[0].payload));
			}
			stopNfcScanner();
		}
		
		function startNfcScanner(){
			if(typeof nfc === 'undefined'){
				return;
			}
			nfc.enabled(function(){
				$scope.nfcReadingEnabled = true;
				nfc.addNdefListener(onTagDiscoevered); // Android only
				$scope.$apply();
			}, function(){
				$scope.nfcReadingEnabled = false;
				modalNotification.alert("NFC isn't enabled. Please enable NFC in device settings.");
				$scope.$apply();
			});
		}
		
		function stopNfcScanner(){
			if(typeof nfc === 'undefined' || !$scope.nfcReadingEnabled){
				return;
			}
			$scope.nfcReadingEnabled = false;
			nfc.removeNdefListener(onTagDiscoevered);
			$scope.$apply();
		}
		
		function scanDataReceived(identifier){
			if($scope.$localStorage.selectedScanPoint.commentsEnabled){
				$scope.latestScannedPerson = dataService.getByID('persons', 'personID', identifier);
				$scope.currentScan = {identifier: identifier, comment: '', scanPointID: $scope.$localStorage.selectedScanPoint.scanPointID};
				$scope.$evalAsync();
			} else {
				$scope.sendScan({identifier: identifier});
			}
		}
		
		$scope.sendScan = function(scanToSave) {
			var scan = angular.copy(scanToSave);
			scan.identifier = encodeURIComponent(scan.identifier);
			var scanData = {scans: [scan]};
			$scope.latestScannedPerson = null;
			$scope.currentScan = null;
			
			$http.post($rootScope.apiUrl + 'scanPoints/' + $scope.$localStorage.selectedScanPoint.scanPointID + '/scans/', scanData).
					then(function (response) {
						console.log(response.data);
						$scope.latestScannedPerson = dataService.getByID('persons', 'personID', scan.identifier);
					}, function (e) {
						if(typeof e.data !== 'undefined' && e.data !== null && typeof e.data.data !== 'undefined' && typeof e.data.data.scans !== 'undefined'){
							var scans = e.data.data.scans;
							if(scans.failed.length > 0){
								var failedScan = {identifier: scan.identifier, scanPointID: $scope.$localStorage.selectedScanPoint.scanPointID, timeScanned: moment().format('YYYY-MM-DDTHH:mm:ssZ')};
								if(scan.comment){
									failedScan.comment = scan.comment;
								}
								var error = typeof scans.failed[0].error !== 'undefined' ? scans.failed[0].error : 'Error! Person not found.';
								failedScan.error = error;
								$scope.$localStorage.failedScans.push(failedScan);
								modalNotification.alert(error);
								return;
							}
						}
						var offlineScan = {identifier: scan.identifier, scanPointID: $scope.$localStorage.selectedScanPoint.scanPointID, timeScanned: moment().format('YYYY-MM-DDTHH:mm:ssZ')};
						if (scan.comment) {
							offlineScan.comment = scan.comment;
						}
						$scope.$localStorage.offlineScans.push(offlineScan);
						$scope.latestScannedPerson = dataService.getByID('persons', 'personID', scan.identifier);
						if($scope.latestScannedPerson === null){
							$scope.latestScannedPerson = {error: "Person wasn't found in offline data. But was stored for later saving."};
						}
						console.log("Error saving scan. Saving to local storage", e);
					});
		};
		
		$scope.openNfcScanner = function(){
			startNfcScanner();
		};
		
		$scope.closeNfcScanner = function(){
			stopNfcScanner();
		};
		
		document.addEventListener("backbutton", onBackKeyDown, false);

		function onBackKeyDown(e) {
			if($scope.qrScannerOpen){
				$scope.qrScannerOpen = false;
				$scope.$apply();
				e.preventDefault();
			}
		}
		
		$scope.$on('$destroy', function () {
			stopNfcScanner();
			document.removeEventListener("backbutton", onBackKeyDown);
		});
	});
})();
