/* global angular */

(function () {

	var app = angular.module('eventosContactApp');

	app.controller("selectScanPointController", function ($scope, $rootScope, $state, $translate, dataService, $localStorage) {
		console.log("selectScanPointController loaded");
		
		$scope.$localStorage = $localStorage;
		
		$scope.itemsToShow = 15;
		$scope.search = $scope.$localStorage.scanPointSearch;
		$scope.showInactiveScanPoints = true;
		
		if(typeof dataService.scanPoints !== 'undefined'){
			$scope.scanPoints = dataService.scanPoints;
		}
		
		dataService.getScanPoints();
		
		$scope.$on('scanPointsLoaded',function(){
			$scope.scanPoints = dataService.scanPoints;
		});
		
		$scope.selectScanPoint = function(scanPoint){
			if($scope.$localStorage.selectedScanPoint !== null && $scope.$localStorage.selectedScanPoint.scanPointID !== scanPoint.scanPointID){
				dataService.clearData('scans');
			}
			$scope.$localStorage.selectedScanPoint = scanPoint;
			$state.go('app.scan');
		};
		
		$scope.searchChanged = function(){
			if(typeof $scope.search === 'string'){
				$scope.itemsToShow = 15;
				$scope.$localStorage.scanPointSearch = $scope.search;
				$scope.showInactiveScanPoints = true;
			}
		};
		
		$scope.isScanPointActive = function(scanPoint){
			var now = moment();
			if(scanPoint.timeActiveStart === null && scanPoint.timeActiveEnd === null){
				return true;
			} else if(scanPoint.timeActiveStart === null && moment(scanPoint.timeActiveEnd) > now){
				return true;
			} else if(scanPoint.timeActiveEnd === null && moment(scanPoint.timeActiveStart) < now){
				return true;
			} else if(moment(scanPoint.timeActiveStart) < now && moment(scanPoint.timeActiveEnd) > now){
				return true;
			} else {
				return false;
			}
		};
		
		$scope.activeScanPointFilter = function () {
			return function (item) {
				if($scope.showInactiveScanPoints){
					return true;
				}
				return $scope.isScanPointActive(item);
			};
		};
		
		function loadMore(){
			if(typeof $scope.scanPoints === "undefined"){
				return;
			}
			if($scope.scanPoints.length > $scope.itemsToShow){
				$scope.itemsToShow += 5;
				$scope.$digest();
			}
		}
		
		var $selectScanPointPage;
		
		function scrolled(e){
			if(typeof $selectScanPointPage === 'undefined'){
				$selectScanPointPage = $('#select-scan-point-page');
			}
			var bottomOfPage = $selectScanPointPage.position().top + $selectScanPointPage.outerHeight(true);
			if(2 * window.innerHeight + window.pageYOffset > bottomOfPage){
				loadMore();
			}
		}
		
		window.addEventListener('scroll', scrolled);
		
		$scope.$on('$destroy', function () {
			window.removeEventListener('scroll', scrolled, false);
		});
	});
})();
