/* global angular */

(function () {

	var app = angular.module('eventosContactApp');

	// Contains app data
	app.service("dataService", ["$http", "$rootScope", "$localStorage", "$translate",
		function ($http, $rootScope, $localStorage, $translate) {

			var server = $rootScope.apiUrl;
			var $storage = $localStorage;

			var that = this;
			this.dataLoading = {
				scanPoints: false,
				events: false,
				scans: false,
				persons: false,
				activityFeed: false,
				sites: false,
				eventBanner: false,
				participantsMetaStatus: false,
			};
			
			this.getByID = function(dataName, dataIDname, searchID){
				if(typeof dataName !== 'string' || typeof that[dataName] === 'undefined' || typeof searchID === 'undefined'){
					return;
				}
				for(var i = 0; i < that[dataName].length; i++){
					if(typeof that[dataName][i] === 'undefined'){
						continue;
					}
					if(that[dataName][i][dataIDname] === searchID){
						return that[dataName][i];
					}
				}
				return null;
			};
			
			this.clearData = function(dataName){
				that[dataName] = null;
				$rootScope.$broadcast(dataName + 'Loaded');
			};

			this.getScanPoints = function (force) {
				if($storage.selectedEvent === null){
					return;
				}
				if (!that.dataLoading.scanPoints || force) {
					that.dataLoading.scanPoints = true;
					$http.get(server + 'events/' + $storage.selectedEvent.eventID + '/scanPoints').
						then(function (response) {
							that.scanPoints = response.data.data.scanPoints;
							that.dataLoading.scanPoints = false;
							$rootScope.$broadcast('scanPointsLoaded');
						}, function (e) {
							that.dataLoading.scanPoints = false;
							console.log("Error fetching scanPoints.", e);
						});
				}
			};

			this.getEvents = function (force) {
				if (!that.dataLoading.events || force) {
					that.dataLoading.events = true;
					if($storage.partner !== null){
						$http.get(server + 'partners/' + $storage.partner.partnerID + '/events').
							then(function (response) {
								that.events = response.data.data.events;
								that.dataLoading.events = false;
								$rootScope.$broadcast('eventsLoaded');
							}, function (e) {
								that.dataLoading.events = false;
								console.log("Error fetching events.", e);
							});
					} else {
						$http.get(server + 'events').
							then(function (response) {
								that.events = response.data.data.events;
								that.dataLoading.events = false;
								$rootScope.$broadcast('eventsLoaded');
							}, function (e) {
								that.dataLoading.events = false;
								console.log("Error fetching events.", e);
							});
					}
				}
			};
			
			this.getPersons = function(force){
				if (!that.dataLoading.persons || force) {
					that.dataLoading.persons = true;
					$http.get(server + 'events/' + $storage.selectedEvent.eventID + '/persons').
							then(function (response) {
								that.persons = response.data.data.persons;
								that.dataLoading.persons = false;
								$rootScope.$broadcast('personsLoaded');
							}, function (e) {
								that.dataLoading.persons = false;
								console.log("Error fetching persons.", e);
							});
				}
			};

			this.getActivityFeed = function(force){
				if (!that.dataLoading.activityFeed || force) {
					that.dataLoading.activityFeed = true;
					$http.get(server + 'events/' + $storage.selectedEvent.eventID + '/activity').
							then(function (response) {
								that.activityFeed = response.data.data.activity;
								that.dataLoading.activityFeed = false;
								$rootScope.$broadcast('activityFeedLoaded');
							}, function (e) {
								that.dataLoading.activityFeed = false;
								console.log("Error fetching activityFeed.", e);
							});
				}
			};
			
			this.getScans = function (force) {
				if (!that.dataLoading.scans || force) {
					that.dataLoading.scans = true;
					$http.get(server + 'scanPoints/' + $storage.selectedScanPoint.scanPointID + '/scans').
						then(function (response) {
							that.scans = response.data.data.scans;
							that.dataLoading.scans = false;
							$rootScope.$broadcast('scansLoaded');
						}, function (e) {
							that.dataLoading.scans = false;
							console.log("Error fetching scans.", e);
						});
				}
			};
			
			this.getParticipantsMetaStatus = function(force){
				if (!that.dataLoading.participantsMetaStatus || force) {
					that.dataLoading.participantsMetaStatus = true;
					$http.get(server + 'persons/meta').
							then(function (response) {
								that.participantsMetaStatus = response.data.data.meta.status;
								that.dataLoading.participantsMetaStatus = false;
								$rootScope.$broadcast('participantsMetaStatusLoaded');
							}, function (e) {
								that.dataLoading.participantsMetaStatus = false;
								console.log("Error fetching participantsMeta.", e);
							});
				}
			};
			
			function getBannerMaterial(materialID, force){
				if (!that.dataLoading.eventBanner || force) {
					that.dataLoading.eventBanner = true;
					$http.get(server + 'materials/' + materialID).
						then(function (response) {
							that.eventBanner = $rootScope.materialUrl + response.data.data.material.absolutePath;
							that.dataLoading.eventBanner = false;
						}, function (e) {
							that.dataLoading.eventBanner = false;
							console.log("Error fetching bannerMaterial.", e);
						});
				}
			}
			
			function getSites (force){
				if (!that.dataLoading.sites || force) {
					that.dataLoading.sites = true;
					$http.get(server + 'events/' + $storage.selectedEvent.eventID + '/sites').
						then(function (response) {
							if(response.data.data.sites.length > 0){
								if(response.data.data.sites[0].bannerMaterialID !== null){
									getBannerMaterial(response.data.data.sites[0].bannerMaterialID);
								}
							}
							that.dataLoading.sites = false;
						}, function (e) {
							that.dataLoading.sites = false;
							console.log("Error fetching sites.", e);
						});
				}
			};
			
			this.getEventBanner = function(){
				if($storage.selectedEvent !== null){
					getSites();
				}
			};

		}]);
})();
