/* global angular */

(function () {

	var app = angular.module('eventosContactApp');

	// Contains app data
	app.service("userService", ["$http", "$rootScope", "$localStorage", "$translate", "jwtHelper", "$q",
		function ($http, $rootScope, $localStorage, $translate, jwtHelper, $q) {

			var server = $rootScope.apiUrl;
			var $storage = $localStorage;

			var that = this;
			
			this.login = function(loginData){
				loginData.service = 'eventosContact';
				return $http.post(server + 'users/login', loginData).
					then(function (response) {
						console.log('Login success', response);
						return response.data;
					}, function (e) {
						console.log("Error trying to login", e);
						return $q.reject(e);
					});
			};
			
			this.getUserData = function(){
				if($storage.jwtToken === null){
					return;
				}
				
				return $http.get(server + 'users/' + this.getUser().userID).
					then(function (response) {
						return response.data.data.user;
					}, function (e) {
						console.log("Error trying to get user data", e);
						return $q.reject(e);
					});
			};
			
			// 2018-08-21 For now this doesn't return all organizations. Tomi Ruusala
			this.getUserOrganizations = function(){
				if($storage.jwtToken === null){
					return;
				}
				
				return $http.get(server + 'organizations').
					then(function (response) {
						return response.data.data;
					}, function (e) {
						console.log("Error trying to get user organizations", e);
						if(typeof e.data.errors !== 'undefined' && e.data.errors.length > 0){
							console.log(e.data.errors);
							return e.data;
						}
					});
			};
			
			this.isAdmin = function(){
				return true;
			};
			
			this.getUser = function () {
				return jwtHelper.decodeToken($storage.jwtToken);
			};

		}]);
})();
