function ModalNotification() {
    this.modals = [];
}

ModalNotification.prototype.alert = function (options) {
    this.modals.push(new AlertModal(options, this));
};

ModalNotification.prototype.confirm = function (options) {
    this.modals.push(new ConfirmModal(options, this));
};

ModalNotification.prototype.confirmTopModal = function () {
    var targetModal = this.modals[this.modals.length - 1];
    if(targetModal instanceof ConfirmModal) {
        targetModal.response = true;
        targetModal.close(true);
    } else if(targetModal instanceof AlertModal) {
        targetModal.close(true);
    }
};

ModalNotification.prototype.removeModal = function (modal) {
    for(var i = 0; i < this.modals.length; i++) {
        if(this.modals[i] === modal) {
            this.modals.splice(i, 1);
        }
    }
	if($('.modal:visible').length > 0){
		$("body").addClass("modal-open");
	}
};

function AlertModal(options, ModalNotification) {
    if(typeof options !== "object" && typeof options !== "string") {
        console.log("Options should be an object or a string, not " + typeof options);
        return false;
    } else if(typeof options === "string") {
        this.text = options;
    } else if(typeof options.text !== "string") {
        console.log("options.text should be a string, not " + typeof options.text);
        return false;
    }
    this.ModalNotification = ModalNotification;
    this.title = options.title || "Notification";
    this.text = this.text || options.text;
    this.okText = options.okText || "OK";
    this.callback = options.callback || function () {
    };
    this.size = options.size || this.text.length > 200 ? "md" : "sm";

    this.element = $("<div class='modal fade modal-notification' data-backdrop='static' data-keyboard='true' tabindex='-1'>\
                        <div class='modal-dialog modal-" + this.size + "'>\
                            <div class='modal-content'>\
                                <div class='modal-header text-center'>\
                                    <h3 class='modal-title'>" + this.title + "</h3>\
                                </div>\
                                <div class='modal-body'>\
                                    <span>" + this.text + "</span>\
                                </div>\
                                <div class='modal-footer'>\
                                    <button type='button' class='btn btn-default' data-dismiss='modal'>" + this.okText + "</button>\
                                </div>\
                            </div>\
                        </div>\
                    </div>");
    $("body").append(this.element);
    var that = this;
    this.element.modal("show").on("hidden.bs.modal", function () {
        that.element.remove();
        that.callback();
        that.close(false);
    });

}

AlertModal.prototype.close = function (dispatch) {
    dispatch = typeof dispatch === "undefined" ? true : dispatch;
    this.element.off("hidden.bs.modal");
    this.element.modal("hide");
    this.element.remove();
    if(dispatch){
        this.callback();
    }
    this.ModalNotification.removeModal(this);
};

function ConfirmModal(options, ModalNotification) {
    if(typeof options !== "object") {
        console.log("Options should be an object, not " + typeof options);
        return false;
    }
    if(typeof options.text !== "string") {
        console.log("options.text should be a string, not " + typeof options.text);
        return false;
    }
    this.ModalNotification = ModalNotification;
    this.title = options.title || "Confirmation";
    this.text = options.text;
    this.okText = options.okText || "OK";
    this.cancelText = options.cancelText || "Cancel";
    this.callback = options.callback || function () {
    };
    this.size = options.size || this.text.length > 200 ? "md" : "sm";

    this.element = $("<div class='modal fade modal-notification' data-backdrop='static' data-keyboard='true' tabindex='-1'>\
                        <div class='modal-dialog modal-" + this.size + "'>\
                            <div class='modal-content'>\
                                <div class='modal-header text-center'>\
                                    <h3 class='modal-title'>" + this.title + "</h3>\
                                </div>\
                                <div class='modal-body'>\
                                    <span>" + this.text + "</span>\
                                </div>\
                                <div class='modal-footer'>\
                                    <button type='button' class='btn btn-default' data-dismiss='modal'>" + this.cancelText + "</button>\
                                    <button type='button' class='btn btn-primary' data-dismiss='modal' onclick='$(this).parents(\".modal\").data(\"ok\")();'>" + this.okText + "</button>\
                                </div>\
                            </div>\
                        </div>\
                    </div>");
    var that = this;
    this.element.data("ok", function () {
        that.response = true;
    });
    $("body").append(this.element);
    this.element.modal("show").on("hidden.bs.modal", function () {
        var response = typeof that.response === "undefined" ? false : that.response;
        that.element.remove();
        that.callback(response);
        that.close(false);
    });
}

ConfirmModal.prototype.close = function (dispatch) {
    dispatch = typeof dispatch === "undefined" ? true : dispatch;
    this.element.off("hidden.bs.modal");
    var that = this;
    this.element.on("hidden.bs.modal", function () {
        this.remove();
    });
    this.element.modal("hide");
    if(dispatch){
        this.callback(that.response);
    }
    this.ModalNotification.removeModal(this);
};